package com.bcxin.web.commons.components;

import com.bcxin.saas.core.components.SecondCacheProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
@Primary
public class SecondCacheProviderRequestAttributeImpl implements SecondCacheProvider {
    private static final Logger logger = LoggerFactory.getLogger(SecondCacheProviderRequestAttributeImpl.class);
    private ThreadLocal<Map<String,Object>> _currentThreadContainer = new InheritableThreadLocal<>();
    private final HttpServletRequest request;

    public SecondCacheProviderRequestAttributeImpl(HttpServletRequest request) {
        this.request = request;
    }

    @Override
    public <T> T getData(String key, Supplier<T> supplier) {
        Object result = getFromCurrentContext(key);

        if (result == null) {
            result = supplier.get();
            set2CurrentContext(key, result);
        }

        return (T) result;
    }

    @Override
    public void removeData(String key) {
        Map<String, Object> data = getSameThreadContainer().get();
        if (data != null && data.containsKey(key)) {
            data.remove(key);

            getSameThreadContainer().set(data);
        }
    }

    @Override
    public <T> T getIfPresent(String key) {
        return (T)getFromCurrentContext(key);
    }

    @Override
    public <T> void setData(String key,T data) {
        try {
            Map<String, Object> map = getSameThreadContainer().get();
            if (map == null) {
                map = new HashMap<>();
                map.put(key, data);

                getSameThreadContainer().set(map);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @Override
    public boolean isWebRequest() {
        try {
            return request != null && StringUtils.hasLength(request.getRequestURI());
        } catch (Exception ex) {
            return false;
        }
    }

    private final static List<String> ignoredRemovedKeys=
            Stream.of("org.","java.","javax.","FILTERED").collect(Collectors.toList());
    @Override
    public void clear() {
        /*
        changed by 20231106 复用线程
        try {
            this._currentThreadContainer.set(null);
        } catch (Exception ex) {
            logger.error("{}: 清除当前请求中的键值失败", new Date());
            ex.printStackTrace();
        }

         */
    }

    private Object getFromCurrentContext(String key) {
        Object result = null;
        Map<String, Object> data = getSameThreadContainer().get();
        if (data == null) {
            data = new HashMap<>();
            getSameThreadContainer().set(data);
        }

        result = data.get(key);

        return result;
    }

    private void set2CurrentContext(String key,Object result)
    {
        Map<String, Object> data = getSameThreadContainer().get();
        if (data == null) {
            data = new HashMap<>();
        }
        if (result == null) {
            data.remove(key);
        } else {
            data.put(key, result);
        }

        getSameThreadContainer().set(data);
    }

    private ThreadLocal<Map<String,Object>> getSameThreadContainer() {
        return _currentThreadContainer;
    }
}
